/* usadelndsoc
 *
 * Copyright © 2022 Pauli Virtanen
 *    @author Pauli Virtanen <pauli.t.virtanen@jyu.fi>
 *
 * SPDX-License-Identifier: AGPL-3.0-or-later
 */
#ifndef ACTION_HPP_
#define ACTION_HPP_

#include "common.hpp"
#include "array.hpp"

template <typename Scalar, typename Shape>
inline auto get_Q_matrix(array::Array<Scalar, Shape> Q)
{
    return 0;
}

template <typename Scalar, typename Shape, typename Shape2>
inline Scalar S_2(array::Array<Scalar, Shape> Q, array::Array<Complex, Shape2> U, double h)
{
    static const int N_i[4] = {0, 1, -1, 0};
    static const int N_j[4] = {1, 0, 0, -1};
    Scalar S = Scalar(0);

    size_t nx = Q.dim(0);
    size_t ny = Q.dim(1);

    for (size_t i = 0; i < nx; ++i) {
        for (size_t j = 0; j < ny; ++j) {
            auto Q1 = Q.part(i,j).matrix();

            /* Sum over neighbors */
            for (size_t k = 0; k < 4; ++k) {
                size_t di = N_i[k];
                size_t dj = N_j[k];
                
                if (i == 0 && di < 0)
                        continue;
                if (i == nx-1 && di > 0)
                        continue;
                if (j == 0 && dj < 0)
                        continue;
                if (j == ny-1 && dj > 0)
                    continue;

                size_t i2 = i + di;
                size_t j2 = j + dj;

                auto Q2 = Q.part(i2,j2).matrix();
                auto U12 = U.part(i,j,k);
                auto U21 = U.part(j,i,k);

                S += 2/(4*h) * (Q1 - U12*Q2*U21).trace();
            }
        }
    }

    return S;
}



#endif

